searchData={"items":[{"type":"module","title":"Moxinet","doc":"`Moxinet` helps you mock the internet at the HTTP layer\nwithout sacrificing parallel testing.","ref":"Moxinet.html"},{"type":"function","title":"Moxinet.build_mock_header/1","doc":"Returns the header needed to be included in requests to the\nmock servers in order to support parallel runs.","ref":"Moxinet.html#build_mock_header/1"},{"type":"function","title":"Moxinet.expect/5","doc":"Mocks a call for the passed module when used from a certain pid, defaulting `self()`","ref":"Moxinet.html#expect/5"},{"type":"function","title":"Moxinet.pid_reference/1","doc":"Turns a pid into a reference which could be used for indexing\nthe signatures.","ref":"Moxinet.html#pid_reference/1"},{"type":"function","title":"Moxinet.start/1","doc":"The start functions which will start the `Moxinet` server.\n\nYou'd most likely want to put run this function from\nyour `test_helper.exs`:\n\n```elixir\n{:ok, _pid} = Moxinet.start(router: MyMockServer, port: 4010)\n```","ref":"Moxinet.html#start/1"},{"type":"function","title":"Options - Moxinet.start/1","doc":"- `router`: A reference to your mock server. *Required*\n  - `port`: The port your mock server will run on. *Required*\n  - `name`: Name of the moxinet supervisor. Defaults to `Moxinet`","ref":"Moxinet.html#start/1-options"},{"type":"type","title":"Moxinet.http_method/0","doc":"","ref":"Moxinet.html#t:http_method/0"},{"type":"module","title":"Moxinet.Mock","doc":"Supports building a custom mock.","ref":"Moxinet.Mock.html"},{"type":"module","title":"Usage - Moxinet.Mock","doc":"While it out of the box will support interactivity\nwith `Moxinet.expect/4`, you'll also be able to treat it like\na Plug and define your own `match` definitions to enable static\nfallbacks. See `Plug.Router` docs for more details.\n\n```elixir\ndefmodule GithubMock do\n  use Moxinet.Mock\n\n  get \"/pull-requests/closed\" do\n    send_resp(conn, 200, [%{id: \"1\", closed: true}])\n  end\nend\n```\n\nSince the mock is just another plug, you can\nchoose to use/build custom plugs to extend its functionality as\na way to add extra verification that your API module does what\nyou want it to do, or to replicate a complex API interaction.\n\n```elixir\ndefmodule GithubMock do\n  use Moxinet.Mock\n\n  import Plug.BasicAuth\n\n  plug :basic_auth, username: \"user\", password: \"s3cr3t_p4s5w0rd\"\nend\n```","ref":"Moxinet.Mock.html#module-usage"},{"type":"module","title":"Moxinet.Response","doc":"A struct to define a response to return from the mock server.","ref":"Moxinet.Response.html"},{"type":"type","title":"Moxinet.Response.t/0","doc":"","ref":"Moxinet.Response.html#t:t/0"},{"type":"module","title":"Moxinet.Server","doc":"Creates a mocking server where service-specific mocks can be added","ref":"Moxinet.Server.html"},{"type":"module","title":"Example - Moxinet.Server","doc":"```elixir\ndefmodule MockServer do\n  use Moxinet.Server\n\n  forward(\"/github\", to: GithubMock)\nend\n```","ref":"Moxinet.Server.html#module-example"},{"type":"macro","title":"Moxinet.Server.__using__/1","doc":"Turns a module into a Moxinet server","ref":"Moxinet.Server.html#__using__/1"},{"type":"macro","title":"Options - Moxinet.Server.__using__/1","doc":"- `log`: Whether requests should be logged or not. Defaults to `false`","ref":"Moxinet.Server.html#__using__/1-options"},{"type":"extras","title":"README","doc":"![GitHub Actions Workflow Status](https://img.shields.io/github/actions/workflow/status/johantell/moxinet/.github%2Fworkflows%2Felixir.yml)\n![Hex.pm Version](https://img.shields.io/hexpm/v/moxinet)\n![Hex.pm License](https://img.shields.io/hexpm/l/moxinet)\n\n# Moxinet\nMocking server that, just like `mox`, allows parallel testing.\n\nHexDocs: https://hexdocs.pm/moxinet","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Install the package by adding `moxinet` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:moxinet, \"~> 0.2.1\", only: :test}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Getting started - README","doc":"To use `moxinet` you must first define your mock server, from which you'll forward\nmock-specific requests:\n\n```elixir\n# test/support/mock_server.ex\n\ndefmodule MyApp.MockServer do\n  use Moxinet.Server\n\n  forward(\"/github\", to: GithubMock)\nend\n```\n\nThen create the mock module (in this example `GithubMock`):\n\n```elixir\n# test/support/mock_servers/github_mock.ex\n\ndefmodule GithubMock do\n  use Moxinet.Mock\nend\n```\n\nStart `moxinet` in your test helper (before `ExUnit.start()`)\n```elixir\n# test/test_helper.exs\n{:ok, _} = Moxinet.start(port: 4040, router: MyApp.MockServer)\n\nExUnit.start()\n```\n\nLet the API configuration decide whether the API should call the remote server or the local mock server:\n\n```elixir\n# config/config.exs\nconfig :my_app, GithubAPI,\n  url: \"https://api.github.com\"\n\n# config/test.exs\nconfig :my_app, GithubAPI,\n  url: \"http://localhost:4040/github\"\n```\n\n---\n\nIf you're familiar with `plug`, you'll see that our mock server is indeed a plug and can therefore\nbe extended like one.\n\nAfter you've added your server, mocks can be defined:\n\n```elixir\n# test/support/mocks/github_mock.ex\ndefmodule GithubMock do\n  use Moxinet.Mock\nend\n```\n\nIn tests, you can create rules for how your mocks should behave through `expect/4`:\n\n```elixir\nalias Moxinet.Response\n\ndescribe \"create_pr/1\" do\n  test \"creates a pull request when\" do\n    GithubMock.expect(:post, \"/pull-requests/123\", fn _payload ->\n      %Response{status: 202, body: %{id: \"pull-request-id\"}}\n    end)\n\n    assert {:ok, %{status: 202, body: %{\"id\" => \"pull-request-id\"}}}} = GithubAPI.create_pr(title: \"My PR\")\n  end\nend\n```\n\n**NOTE**: One small caveat with `moxinet` is that in order for us to be able to match\na mock defined in a request with an incoming request, the requests must send the `x-moxinet-ref` header.\nMost HTTP libraries allows adding custom headers to your requests, but that might not always be the case.\n\nIt's also not recommended to send the `x-moxinet-ref` header outside your test environment, meaning you'd\nlikely want to do find a way to conditionally add it. Here is one example on how to achieve it in `req`:\n\n```elixir\ndefmodule GithubAPI do\n  def client do\n    Req.new([\n      # ...\n    ])\n    |> add_moxinet_header()\n  end\n\n  defmacrop add_moxinet_header(req) do\n    if Mix.env() == :test do\n      quote do\n        {header_name, header_value} = Moxinet.build_mock_header()\n      \n        Req.Request.put_new_header(unquote(req), header_name, header_value)\n      end\n    else\n      quote do\n        unquote(req)\n      end\n    end\n  end\nend\n```","ref":"readme.html#getting-started"},{"type":"extras","title":"Why not use `mox` instead? - README","doc":"When testing calls to external servers `mox` tends to guide the user towards\nreplacing the whole HTTP layer which is usually fine when the code is controlled\nby an external library, holding the responsibility of correctness on its own.\n\nA commonly seen pattern where behaviors play the centric role like the following:\n\n```elixir\ndefmodule GithubAPI do\n  defmodule HTTPBehaviour do\n    @callback post(String.t(), Keyword.t()) :: {:ok, Map.t()} | {:error, :atom}\n  end\n\n  defmodule HTTP do\n    @behaviour GithubAPI.HTTPBehaviour\n    def post(url, opts) do\n      # Perform HTTP request\n    end\n  end\n\n  def create_pr(attrs) do\n    impl().post(\"/pull-requests\", body: attrs)\n  end\n\n  defp impl, do: Application.get_env(:github_api_http_module, HTTP)\nend\n```\n\nwill absolutely do the job, but there are a few drawbacks:\n\n1. The `HTTP` remains untested as that is not being used by the test suite\n2. HTTP client libraries (like tesla) usually allow defining headers, authentication,\n   and in most cases also encodes passed data structures to JSON, where custom behavior\n   that filters/encodes data in certain ways can be included. I've seen cases where a\n   `@derive {Jason, only: [...]}` caused a bug that wasn't a bug according to all tests\n   that verified the expected data was sent to the HTTP layer.\n\n`moxinet` aims to fill those gaps while also reducing the need for behaviors and mocks","ref":"readme.html#why-not-use-mox-instead"},{"type":"extras","title":"How it works - README","doc":"`Moxinet` works a lot similar to `mox` except it'll focus on solving the same issue for HTTP request.\n\nThe test pid will be registered in the mock registry, where it can later be accessed from inside the mock.\n\n```mermaid\nflowchart TD\n    TP[Test pid] --> MR[Mock registry]\n    GMS <--> MR\n    TP --> API[Github API]\n    API -.HTTP request.-> MS[Mock server]\n    MS --> GMS[Github Mock]\n    GMS -.HTTP response.-> API\n```","ref":"readme.html#how-it-works"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}